@@ -125,7 +125,8 @@ static void gconf_client_finalize              (GObject* object);
 
 static gboolean gconf_client_cache          (GConfClient *client,
                                              gboolean     take_ownership,
-                                             GConfEntry  *entry);
+                                             GConfEntry  *entry,
+                                             gboolean    preserve_schema_name);
 
 static gboolean gconf_client_lookup         (GConfClient *client,
                                              const char  *key,
@@ -340,8 +341,8 @@ gconf_client_real_unreturned_error (GConfClient* client, GError* error)
               error->code == GCONF_ERROR_NO_WRITABLE_DATABASE)
             return;
           
-          fprintf (stderr, _("GConf Error: %s\n"),
-                   error->message);
+          g_printerr (_("GConf Error: %s\n"),
+                      error->message);
         }
     }
 }
@@ -358,8 +359,8 @@ gconf_client_real_error            (GConfClient* client, GError* error)
         }
       else
         {
-          fprintf (stderr, _("GConf Error: %s\n"),
-                   error->message);
+          g_printerr (_("GConf Error: %s\n"),
+                      error->message);
         }
     }
 }
@@ -406,7 +407,7 @@ notify_from_server_callback (GConfEngine* conf, guint cnxn_id,
    * listeners or functions connected to value_changed.
    * We know this key is under a directory in our dir list.
    */
-  changed = gconf_client_cache (client, FALSE, entry);                                
+  changed = gconf_client_cache (client, FALSE, entry, TRUE);
 
   if (!changed)
     return; /* don't do the notify */
@@ -545,7 +546,7 @@ gconf_client_add_dir     (GConfClient* client,
 
   g_return_if_fail (gconf_valid_key (dirname, NULL));
 
-  trace ("Adding dir '%s'\n", dirname);
+  trace ("Adding directory '%s'\n", dirname);
   
   d = g_hash_table_lookup (client->dir_hash, dirname);
 
@@ -683,7 +684,7 @@ gconf_client_remove_dir  (GConfClient* client,
 {
   Dir* found = NULL;
 
-  trace ("Removing dir '%s'\n", dirname);
+  trace ("Removing directory '%s'\n", dirname);
   
   found = g_hash_table_lookup (client->dir_hash,
                                dirname);
@@ -775,7 +776,6 @@ gconf_client_set_error_handling(GConfClient* client,
 static gboolean
 clear_cache_foreach (char* key, GConfEntry* entry, GConfClient* client)
 {
-  g_free (key);
   gconf_entry_free (entry);
 
   return TRUE;
@@ -799,7 +799,7 @@ static void
 cache_pairs_in_dir(GConfClient* client, const gchar* path);
 
 static void 
-recurse_subdir_list(GConfClient* client, GSList* subdirs, const gchar* parent)
+recurse_subdir_list(GConfClient* client, GSList* subdirs)
 {
   GSList* tmp;
 
@@ -808,18 +808,15 @@ recurse_subdir_list(GConfClient* client, GSList* subdirs, const gchar* parent)
   while (tmp != NULL)
     {
       gchar* s = tmp->data;
-      gchar* full = gconf_concat_dir_and_key(parent, s);
       
-      cache_pairs_in_dir(client, full);
+      cache_pairs_in_dir(client, s);
 
       PUSH_USE_ENGINE (client);
       recurse_subdir_list(client,
-                          gconf_engine_all_dirs (client->engine, full, NULL),
-                          full);
+                          gconf_engine_all_dirs (client->engine, s, NULL));
       POP_USE_ENGINE (client);
 
       g_free(s);
-      g_free(full);
       
       tmp = g_slist_next(tmp);
     }
@@ -873,7 +870,7 @@ cache_entry_list_destructively (GConfClient *client,
     {
       GConfEntry* entry = tmp->data;
       
-      gconf_client_cache (client, TRUE, entry);
+      gconf_client_cache (client, TRUE, entry, FALSE);
       
       tmp = g_slist_next (tmp);
     }
@@ -895,8 +892,8 @@ cache_pairs_in_dir(GConfClient* client, const gchar* dir)
   
   if (error != NULL)
     {
-      fprintf(stderr, _("GConf warning: failure listing pairs in `%s': %s"),
-              dir, error->message);
+      g_printerr (_("GConf warning: failure listing pairs in `%s': %s"),
+                  dir, error->message);
       g_error_free(error);
       error = NULL;
     }
@@ -932,14 +929,8 @@ gconf_client_preload    (GConfClient* client,
 
     case GCONF_CLIENT_PRELOAD_ONELEVEL:
       {
-        GSList* subdirs;
-
         trace ("Onelevel preload of '%s'\n", dirname);
         
-        PUSH_USE_ENGINE (client);
-        subdirs = gconf_engine_all_dirs (client->engine, dirname, NULL);
-        POP_USE_ENGINE (client);
-        
         cache_pairs_in_dir (client, dirname);
       }
       break;
@@ -956,7 +947,7 @@ gconf_client_preload    (GConfClient* client,
         
         cache_pairs_in_dir(client, dirname);
           
-        recurse_subdir_list(client, subdirs, dirname);
+        recurse_subdir_list(client, subdirs);
       }
       break;
 
@@ -1120,7 +1111,7 @@ gconf_client_dir_exists(GConfClient* client,
   GError* error = NULL;
   gboolean retval;
 
-  trace ("Checking whether dir '%s' exists...\n", dir);
+  trace ("Checking whether directory '%s' exists...\n", dir);
   
   PUSH_USE_ENGINE (client);
   retval = gconf_engine_dir_exists (client->engine, dir, &error);
@@ -1247,7 +1238,7 @@ get (GConfClient *client,
       if (key_being_monitored (client, key))
         {
           /* cache a copy of val */
-          gconf_client_cache (client, FALSE, entry);
+          gconf_client_cache (client, FALSE, entry, FALSE);
         }
 
       /* We don't own the entry, we're returning this copy belonging
@@ -1890,7 +1881,8 @@ gconf_client_value_changed          (GConfClient* client,
 static gboolean
 gconf_client_cache (GConfClient *client,
                     gboolean     take_ownership,
-                    GConfEntry  *new_entry)
+                    GConfEntry  *new_entry,
+                    gboolean     preserve_schema_name)
 {
   gpointer oldkey = NULL, oldval = NULL;
 
@@ -1912,6 +1904,10 @@ gconf_client_cache (GConfClient *client,
           if (!take_ownership)
             new_entry = gconf_entry_copy (new_entry);
           
+          if (preserve_schema_name)
+            gconf_entry_set_schema_name (new_entry, 
+                                         gconf_entry_get_schema_name (entry));
+
           g_hash_table_replace (client->cache_hash,
                                 new_entry->key,
                                 new_entry);
